@@ -7,7 +7,7 @@ This sole data augmentation was proposed by Pasa in the article "Efficient Deep
 Network Architectures for Fast Chest X-Ray Tuberculosis Screening and
 Visualization".
 
-Reference: [PASA-2019]_
+Reference: :cite:p:`pasa_efficient_2019`
 """
 
 from ...data.augmentations import ElasticDeformation
