@@ -7,7 +7,7 @@ Implementation of the model architecture proposed by F. Pasa in the article
 "Efficient Deep Network Architectures for Fast Chest X-Ray Tuberculosis
 Screening and Visualization".
 
-Reference: [PASA-2019]_
+Reference: :cite:p:`pasa_efficient_2019`
 """
 
 import torch.nn
