@@ -8,19 +8,19 @@ Deep Retinal Image Understanding (DRIU), a unified framework of retinal image
 analysis that provides both retinal vessel and optic disc segmentation using
 deep Convolutional Neural Networks (CNNs).
 
-Reference: [MANINIS-2016]_
+Reference: :cite:p:`maninis_deep_2016`
 """
 
 import torch.optim
 import torchvision.transforms
 import torchvision.transforms.v2
 
+import mednet.models.losses
 import mednet.models.segment.driu
-import mednet.models.segment.losses
 import mednet.models.transforms
 
 model = mednet.models.segment.driu.DRIU(
-    loss_type=mednet.models.segment.losses.SoftJaccardBCELogitsLoss,
+    loss_type=mednet.models.losses.SoftJaccardAndBCEWithLogitsLoss,
     loss_arguments=dict(alpha=0.7),  # 0.7 BCE + 0.3 Jaccard
     optimizer_type=torch.optim.Adam,
     optimizer_arguments=dict(lr=0.01),
