@@ -4,19 +4,19 @@
 
 """HED Network for Segmentation.
 
-Reference: [XIE-2015]_
+Reference: :cite:p:`xie_holistically-nested_2015`
 """
 
 import torch.optim
 import torchvision.transforms
 import torchvision.transforms.v2
 
+import mednet.models.losses
 import mednet.models.segment.hed
-import mednet.models.segment.losses
 import mednet.models.transforms
 
 model = mednet.models.segment.hed.HED(
-    loss_type=mednet.models.segment.losses.MultiSoftJaccardBCELogitsLoss,
+    loss_type=mednet.models.losses.MultiLayerSoftJaccardAndBCELogitsLoss,
     loss_arguments=dict(alpha=0.7),  # 0.7 BCE + 0.3 Jaccard
     optimizer_type=torch.optim.Adam,
     optimizer_arguments=dict(lr=0.01),
