@@ -6,7 +6,7 @@
 The Little W-Net architecture contains roughly around 70k parameters and
 closely matches (or outperforms) other more complex techniques.
 
-Reference: [GALDRAN-2020]_
+Reference: :cite:p:`galdran_state---art_2022`
 """
 
 import torch.optim
@@ -14,7 +14,7 @@ import torch.optim.lr_scheduler
 import torchvision.transforms
 import torchvision.transforms.v2
 
-import mednet.models.segment.losses
+import mednet.models.losses
 import mednet.models.segment.lwnet
 import mednet.models.transforms
 
@@ -25,7 +25,7 @@ min_lr = 1e-08  # valley
 cycle = 100  # 1/2 epochs for a complete scheduling cycle
 
 model = mednet.models.segment.lwnet.LittleWNet(
-    loss_type=mednet.models.segment.losses.MultiWeightedBCELogitsLoss,
+    loss_type=mednet.models.losses.MultiLayerBCELogitsLossWeightedPerBatch,
     loss_arguments=dict(),
     optimizer_type=torch.optim.Adam,
     optimizer_arguments=dict(lr=max_lr),
