@@ -12,19 +12,19 @@ the intermediate expansion layer.  This model implements a MobileNetV2 U-Net
 model, henceforth named M2U-Net, combining the strenghts of U-Net for medical
 segmentation applications and the speed of MobileNetV2 networks.
 
-References: [SANDLER-2018]_, [RONNEBERGER-2015]_
+References: :cite:p:`sandler_mobilenetv2_2018`, :cite:p:`navab_u-net_2015`
 """
 
 import torch.optim
 import torchvision.transforms
 import torchvision.transforms.v2
 
-import mednet.models.segment.losses
+import mednet.models.losses
 import mednet.models.segment.m2unet
 import mednet.models.transforms
 
 model = mednet.models.segment.m2unet.M2Unet(
-    loss_type=mednet.models.segment.losses.SoftJaccardBCELogitsLoss,
+    loss_type=mednet.models.losses.SoftJaccardAndBCEWithLogitsLoss,
     loss_arguments=dict(alpha=0.7),  # 0.7 BCE + 0.3 Jaccard
     optimizer_type=torch.optim.Adam,
     optimizer_arguments=dict(lr=0.01),
