@@ -10,19 +10,19 @@ Germany.  The network is based on the fully convolutional network (FCN) and its
 architecture was modified and extended to work with fewer training images and
 to yield more precise segmentations.
 
-Reference: [RONNEBERGER-2015]_
+Reference: :cite:p:`navab_u-net_2015`
 """
 
 import torch.optim
 import torchvision.transforms
 import torchvision.transforms.v2
 
-import mednet.models.segment.losses
+import mednet.models.losses
 import mednet.models.segment.unet
 import mednet.models.transforms
 
 model = mednet.models.segment.unet.Unet(
-    loss_type=mednet.models.segment.losses.SoftJaccardBCELogitsLoss,
+    loss_type=mednet.models.losses.SoftJaccardAndBCEWithLogitsLoss,
     loss_arguments=dict(alpha=0.7),  # 0.7 BCE + 0.3 Jaccard
     optimizer_type=torch.optim.Adam,
     optimizer_arguments=dict(lr=0.01),
