@@ -6,15 +6,16 @@
 import functools
 import itertools
 import logging
-import multiprocessing
 import sys
 import typing
 
 import lightning
+import loky
 import torch
 import torch.backends
 import torch.utils.data
 import torchvision.transforms
+import torchvision.tv_tensors
 import tqdm
 
 from .typing import (
@@ -24,7 +25,6 @@ from .typing import (
     Dataset,
     RawDataLoader,
     Sample,
-    Transform,
     TransformSequence,
 )
 
@@ -40,13 +40,15 @@ def _sample_size_bytes(dataset: Dataset):
         The dataset containing the samples to load.
     """
 
-    def _tensor_size_bytes(t: torch.Tensor) -> int:
+    def _tensor_size_bytes(t: torch.Tensor, n: str) -> int:
         """Return a tensor size in bytes.
 
         Parameters
         ----------
         t
             A torch Tensor.
+        n
+            Name of the object.
 
         Returns
         -------
@@ -54,7 +56,7 @@ def _sample_size_bytes(dataset: Dataset):
             The size of the Tensor in bytes.
         """
 
-        logger.info(f"{list(t.shape)}@{t.dtype}")
+        logger.info(f"`{n}`: {list(t.shape)}@{t.dtype}")
         return int(t.element_size() * t.shape.numel())
 
     def _dict_size_bytes(d):
@@ -72,27 +74,42 @@ def _sample_size_bytes(dataset: Dataset):
         """
 
         size = 0
-        for v in d.values():
+        for k, v in d.items():
             if isinstance(v, torch.Tensor):
-                size += _tensor_size_bytes(v)
+                size += _tensor_size_bytes(v, k)
 
         return size
 
-    size = 0
-
     first_sample = dataset[0]
-
-    for s in first_sample:
-        size += sys.getsizeof(s)
-        if isinstance(s, dict):
-            size += _dict_size_bytes(s)
-        else:
-            size += _tensor_size_bytes(s)
+    size = sys.getsizeof(first_sample)  # measures size of all pythonic objects
+    size += _dict_size_bytes(first_sample)  # adds torch tensor sizes
 
     sample_size_mb = size / (1024.0 * 1024.0)
     logger.info(f"Estimated sample size: {sample_size_mb:.1f} Mb")
 
 
+def _apply_iff_tv_tensor(
+    data: typing.Any, transform: typing.Callable[[torch.Tensor], torch.Tensor]
+):
+    """Apply model_transform iff input object is TVTensor.
+
+    Parameters
+    ----------
+    data
+        Sample data to which apply the provided transform.
+    transform
+        Callable containing a single or a composition of transforms to
+        potentially apply to ``data``.
+
+    Returns
+    -------
+        The transformed version of ``data`` iff applicable.
+    """
+    if isinstance(data, torchvision.tv_tensors.TVTensor):
+        return transform(data)
+    return data
+
+
 class _DelayedLoadingDataset(Dataset):
     """A list that loads its samples on demand.
 
@@ -109,6 +126,8 @@ class _DelayedLoadingDataset(Dataset):
     transforms
         A set of transforms that should be applied on-the-fly for this dataset,
         to fit the output of the raw-data-loader to the model of interest.
+    disable_pbar
+        If set, disables progress bars.
     """
 
     def __init__(
@@ -116,16 +135,33 @@ class _DelayedLoadingDataset(Dataset):
         raw_dataset: typing.Sequence[typing.Any],
         loader: RawDataLoader,
         transforms: TransformSequence = [],
+        disable_pbar: bool = False,
     ):
         self.raw_dataset = raw_dataset
         self.loader = loader
         self.transform = torchvision.transforms.Compose(transforms)
+        self.disable_pbar = disable_pbar
 
         _sample_size_bytes(self)
 
+    def targets(self) -> list[torch.Tensor]:
+        """Return the targets for all samples in the dataset.
+
+        Returns
+        -------
+            The targets for all samples in the dataset.
+        """
+
+        return [
+            self.loader.target(k)
+            for k in tqdm.tqdm(
+                self.raw_dataset, unit="sample", disable=self.disable_pbar
+            )
+        ]
+
     def __getitem__(self, key: int) -> Sample:
-        tensor, metadata = self.loader.sample(self.raw_dataset[key])
-        return self.transform(tensor), metadata
+        sample = self.loader.sample(self.raw_dataset[key])
+        return {k: _apply_iff_tv_tensor(v, self.transform) for k, v in sample.items()}
 
     def __len__(self):
         return len(self.raw_dataset)
@@ -159,12 +195,11 @@ def _apply_loader_and_transforms(
     Sample
         The loaded and transformed sample.
     """
-
     sample = load(info)
-    return model_transform(sample[0]), sample[1]
+    return {k: _apply_iff_tv_tensor(v, model_transform) for k, v in sample.items()}
 
 
-class _CachedDataset(Dataset):
+class CachedDataset(Dataset):
     """Basically, a list of preloaded samples.
 
     This dataset will load all samples from the raw dataset during construction
@@ -178,23 +213,26 @@ class _CachedDataset(Dataset):
         database split datasets.
     loader
         An object instance that can load samples and targets from storage.
+    transforms
+        A set of transforms that should be applied to the cached samples for
+        this dataset, to fit the output of the raw-data-loader to the model of
+        interest.
     parallel
         Use multiprocessing for data loading: if set to -1 (default), disables
         multiprocessing data loading.  Set to 0 to enable as many data loading
         instances as processing cores available in the system.  Set to >= 1
         to enable that many multiprocessing instances for data loading.
-    transforms
-        A set of transforms that should be applied to the cached samples for
-        this dataset, to fit the output of the raw-data-loader to the model of
-        interest.
+    disable_pbar
+        If set, disables progress bars.
     """
 
     def __init__(
         self,
         raw_dataset: typing.Sequence[typing.Any],
         loader: RawDataLoader,
-        parallel: int = -1,
         transforms: TransformSequence = [],
+        parallel: int = -1,
+        disable_pbar: bool = False,
     ):
         self.loader = functools.partial(
             _apply_loader_and_transforms,
@@ -203,30 +241,42 @@ class _CachedDataset(Dataset):
         )
 
         if parallel < 0:
-            self.data = [self.loader(k) for k in tqdm.tqdm(raw_dataset, unit="sample")]
+            self.data = [
+                self.loader(k)
+                for k in tqdm.tqdm(raw_dataset, unit="sample", disable=disable_pbar)
+            ]
         else:
-            instances = parallel or multiprocessing.cpu_count()
+            instances = parallel or torch.multiprocessing.cpu_count()
             logger.info(f"Caching dataset using {instances} processes...")
-            with multiprocessing.Pool(instances) as p:
+            # loky executor replaces torch.multiprocessing.Pool, but uses cloudpickle
+            # for more robust data exchange between main process and workers.
+            with loky.ProcessPoolExecutor(max_workers=instances) as executor:
+                # submit all tasks
+                _ = {
+                    executor.submit(self.loader, sample): sample
+                    for sample in raw_dataset
+                }
+                mapped = executor.map(self.loader, raw_dataset)
                 self.data = list(
                     tqdm.tqdm(
-                        p.imap(self.loader, raw_dataset),
+                        mapped,
                         total=len(raw_dataset),
-                    ),
+                        disable=disable_pbar,
+                        unit="sample",
+                    )
                 )
 
         _sample_size_bytes(self)
 
-    def targets(self) -> list[int | list[int]]:
-        """Return the integer targets for all samples in the dataset.
+    def targets(self) -> list[torch.Tensor]:
+        """Return the targets for all samples in the dataset.
 
         Returns
         -------
-        list[int | list[int]]
-            The integer targets for all samples in the dataset.
+            The targets for all samples in the dataset.
         """
 
-        return [k[1]["target"] for k in self.data]
+        return [k["target"] for k in self.data]
 
     def __getitem__(self, key: int) -> Sample:
         return self.data[key]
@@ -238,9 +288,8 @@ class _CachedDataset(Dataset):
         yield from self.data
 
 
-class _ConcatDataset(Dataset):
-    """A dataset that represents a concatenation of other cached or delayed
-    datasets.
+class ConcatDataset(Dataset):
+    """A dataset that represents a concatenation of other cached or delayed datasets.
 
     Parameters
     ----------
@@ -256,13 +305,12 @@ class _ConcatDataset(Dataset):
             for j in range(len(datasets[i]))
         ]
 
-    def targets(self) -> list[int | list[int]]:
-        """Return the integer targets for all samples in the dataset.
+    def targets(self) -> list[torch.Tensor]:
+        """Return the targets for all samples in the dataset.
 
         Returns
         -------
-        list[int | list[int]]
-            The integer targets for all samples in the dataset.
+            The targets for all samples in the dataset.
         """
 
         return list(itertools.chain(*[k.targets() for k in self._datasets]))
@@ -301,13 +349,13 @@ class ConcatDataModule(lightning.LightningDataModule):
         A dictionary that contains string keys representing dataset names, and
         values that are iterables over a 2-tuple containing an iterable over
         arbitrary, user-configurable sample representations (potentially on
-        disk or permanent storage), and :py:class:`.typing.RawDataLoader` (or
-        "sample") loader objects, which concretely implement a mechanism to
+        disk or permanent storage), and :py:class:`.data.typing.RawDataLoader`
+        (or "sample") loader objects, which concretely implement a mechanism to
         load such samples in memory, from permanent storage.
 
         Sample representations on permanent storage may be of any iterable
         format (e.g. list, dictionary, etc.), for as long as the assigned
-        :py:class:`.typing.RawDataLoader` can properly handle it.
+        :py:class:`.data.typing.RawDataLoader` can properly handle it.
 
         .. tip::
 
@@ -326,8 +374,13 @@ class ConcatDataModule(lightning.LightningDataModule):
         The name of the split used to group the samples into the various
         datasets for training, validation and testing.
     task
-        The task this datamodule generate samples for (e.g. ``classification``
-        or ``segmentation``).
+        The task this datamodule generate samples for (e.g. ``classification``,
+        ``segmentation``, or ``detection``).
+    num_classes
+        The number of target classes samples of this datamodule can have. In a
+        classification task, this will dictate the number of outputs for the classifier
+        (one-hot-encoded), the number of segmentation outputs for a semantic
+        segmentation network, or the types of objects in an object detector.
     collate_fn
         A custom function to batch the samples.
         Uses torch.utils.data.default_collate() by default.
@@ -370,6 +423,7 @@ class ConcatDataModule(lightning.LightningDataModule):
         database_name: str = "",
         split_name: str = "",
         task: str = "",
+        num_classes: int = 1,
         collate_fn=torch.utils.data.default_collate,
         cache_samples: bool = False,
         batch_size: int = 1,
@@ -382,6 +436,7 @@ class ConcatDataModule(lightning.LightningDataModule):
         self.database_name = database_name
         self.split_name = split_name
         self.task = task
+        self.num_classes = num_classes
         self.collate_fn = collate_fn
 
         for dataset_name, split_loaders in splits.items():
@@ -393,7 +448,7 @@ class ConcatDataModule(lightning.LightningDataModule):
 
         self.cache_samples = cache_samples
 
-        self._model_transforms: list[Transform] | None = None
+        self._model_transforms: TransformSequence | None = None
 
         self.batch_size = batch_size
 
@@ -418,8 +473,9 @@ class ConcatDataModule(lightning.LightningDataModule):
         instances for data loading.
 
         It sets the parameter ``num_workers`` (from DataLoaders) to match the
-        expected pytorch representation.  For macOS machines, it also sets the
-        ``multiprocessing_context`` to use ``spawn`` instead of the default.
+        expected pytorch representation.  It also sets the
+        ``multiprocessing_context`` to use ``spawn`` instead of the default
+        (``fork``, on Linux).
 
         The mapping between the command-line interface ``parallel`` setting
         works like this:
@@ -460,21 +516,19 @@ class ConcatDataModule(lightning.LightningDataModule):
             num_workers = 0
 
         else:
-            num_workers = value or multiprocessing.cpu_count()
+            num_workers = value or torch.multiprocessing.cpu_count()
 
         self._dataloader_multiproc["num_workers"] = num_workers
 
-        if num_workers > 0 and sys.platform == "darwin":
-            self._dataloader_multiproc["multiprocessing_context"] = (
-                multiprocessing.get_context("spawn")
-            )
+        if num_workers > 0:
+            self._dataloader_multiproc["multiprocessing_context"] = "spawn"
 
         # keep workers hanging around if we have multiple
         if value >= 0:
             self._dataloader_multiproc["persistent_workers"] = True
 
     @property
-    def model_transforms(self) -> list[Transform] | None:
+    def model_transforms(self) -> TransformSequence | None:
         """Transform required to fit data into the model.
 
         A list of transforms (torch modules) that will be applied after
@@ -495,7 +549,7 @@ class ConcatDataModule(lightning.LightningDataModule):
         return self._model_transforms
 
     @model_transforms.setter
-    def model_transforms(self, value: list[Transform] | None):
+    def model_transforms(self, value: TransformSequence | None):
         old_value = self._model_transforms
         if value is None:
             self._model_transforms = value
@@ -529,11 +583,11 @@ class ConcatDataModule(lightning.LightningDataModule):
 
         if name in self._datasets:
             logger.info(
-                f"Dataset `{name}` is already setup. " f"Not re-instantiating it.",
+                f"Dataset `{name}` is already setup. Not re-instantiating it.",
             )
             return
 
-        datasets: list[_CachedDataset | _DelayedLoadingDataset] = []
+        datasets: list[CachedDataset | _DelayedLoadingDataset] = []
         if self.cache_samples:
             logger.info(
                 f"Loading dataset:`{name}` into memory (caching)."
@@ -541,12 +595,7 @@ class ConcatDataModule(lightning.LightningDataModule):
             )
             for split, loader in self.splits[name]:
                 datasets.append(
-                    _CachedDataset(
-                        split,
-                        loader,
-                        self.parallel,
-                        self.model_transforms,
-                    ),
+                    CachedDataset(split, loader, self.model_transforms, self.parallel)
                 )
         else:
             logger.info(
@@ -555,17 +604,13 @@ class ConcatDataModule(lightning.LightningDataModule):
             )
             for split, loader in self.splits[name]:
                 datasets.append(
-                    _DelayedLoadingDataset(
-                        split,
-                        loader,
-                        self.model_transforms,
-                    ),
+                    _DelayedLoadingDataset(split, loader, self.model_transforms)
                 )
 
         if len(datasets) == 1:
             self._datasets[name] = datasets[0]
         else:
-            self._datasets[name] = _ConcatDataset(datasets)
+            self._datasets[name] = ConcatDataset(datasets)
 
     def val_dataset_keys(self) -> list[str]:
         """Return list of validation dataset names.
@@ -764,7 +809,7 @@ class CachingDataModule(ConcatDataModule):
         A dictionary that contains string keys representing dataset names, and
         values that are iterables over sample representations (potentially on
         disk).  These objects are passed to an unique
-        :py:class:`.typing.RawDataLoader` for loading the
+        :py:class:`.data.typing.RawDataLoader` for loading the
         :py:data:`.typing.Sample` data (and metadata) in memory.  It
         therefore assumes the whole split is homogeneous and can be loaded in
         the same way.
