@@ -5,26 +5,12 @@
 
 import typing
 
-BinaryPrediction: typing.TypeAlias = tuple[str, int, float]
+import timm
+
+Prediction: typing.TypeAlias = tuple[str, typing.Sequence[int], typing.Sequence[float]]
 """The sample name, the target, and the predicted value."""
 
-MultiClassPrediction: typing.TypeAlias = tuple[
-    str,
-    typing.Sequence[int],
-    typing.Sequence[float],
-]
-"""The sample name, the target, and the predicted value."""
-
-BinaryPredictionSplit: typing.TypeAlias = typing.Mapping[
-    str,
-    typing.Sequence[BinaryPrediction],
-]
-"""A series of predictions for different database splits."""
-
-MultiClassPredictionSplit: typing.TypeAlias = typing.Mapping[
-    str,
-    typing.Sequence[MultiClassPrediction],
-]
+PredictionSplit: typing.TypeAlias = typing.Mapping[str, typing.Sequence[Prediction]]
 """A series of predictions for different database splits."""
 
 SaliencyMapAlgorithm: typing.TypeAlias = typing.Literal[
@@ -43,3 +29,10 @@ SaliencyMapAlgorithm: typing.TypeAlias = typing.Literal[
     "xgradcam",
 ]
 """Supported saliency map algorithms."""
+
+architecture_names = sorted(
+    timm.list_models(filter="vit*", module="vision_transformer")
+    + timm.list_models(filter="vit*", module="vision_transformer", include_tags=True)
+)
+ViTArchitectures: typing.TypeAlias = typing.Literal[*architecture_names]  # type: ignore
+"""Names of supported ViT architectures in timm."""
