@@ -24,6 +24,7 @@ def _add_command(module, obj):
 
 
 _add_command(".info", "info")
+_add_command(".info", "version")
 _add_command(".config", "config")
 _add_command(".database", "database")
 _add_command(".train", "train")
@@ -32,4 +33,5 @@ _add_command(".predict", "predict")
 _add_command(".experiment", "experiment")
 _add_command(".upload", "upload")
 _add_command(".classify.cli", "classify")
+_add_command(".detect.cli", "detect")
 _add_command(".segment.cli", "segment")
