@@ -472,7 +472,7 @@ def _monitor_worker(
             if wait_at_most < 0:
                 warnings.warn(
                     f"The time to run a monitor.log() call "
-                    f"({(time.time()-start):.2f}s) is larger than the "
+                    f"({(time.time() - start):.2f}s) is larger than the "
                     f"interval of {interval}s. The amount of time between "
                     f"monitor log calls may be larger than the estimated "
                     f"interval time."
@@ -539,6 +539,9 @@ class ResourceMonitor:
         if not self.monitor.is_alive():
             timeout = 60  # seconds
             self.monitor.start()
+            logger.info(
+                f"Started resource monitoring process (PID: {self.monitor.pid})"
+            )
             try:
                 obj = self.q.get(timeout=timeout)
                 if obj != {}:
@@ -557,18 +560,47 @@ class ResourceMonitor:
         """Start the monitoring process."""
         self.start()
 
-    def stop(self) -> None:
-        """Stop the monitoring process."""
+    def stop(self, timeout: float = 5.0) -> None:
+        """Stop the monitoring process (hard-kill after *timeout* seconds).
 
-        if self.monitor.is_alive():
-            self.stop_event.set()
-            self.wakeup_event.set()
-            self.monitor.join()
-            if self.monitor.exitcode != 0:
-                logger.error(
-                    f"CPU/GPU resource monitor process exited with code "
-                    f"{self.monitor.exitcode}.  Check logs for errors!",
-                )
+        Parameters
+        ----------
+        timeout
+            Timeout for joining the main process.
+        """
+
+        logger.debug(f"Stopping resource monitoring process (PID: {self.monitor.pid})")
+
+        if not self.monitor.is_alive():
+            return
+
+        # tell the child to exit and wake it up
+        self.stop_event.set()
+        self.wakeup_event.set()
+
+        self.monitor.join(timeout)
+        if self.monitor.is_alive():  # still hanging? kill it.
+            logger.warning("ResourceMonitor: soft stop timed out – forcing terminate()")
+            self.monitor.terminate()
+            self.monitor.join(2.0)
+
+        # be nice to the queue – avoid GC deadlocks at interpreter shutdown
+        try:
+            self.q.close()
+            self.q.join_thread()
+        except (OSError, ValueError):
+            pass
+
+        if self.monitor.exitcode != 0:
+            logger.error(
+                f"CPU/GPU resource monitor process exited with code "
+                f"{self.monitor.exitcode}.  Check logs for errors!",
+            )
+
+        logger.info(
+            f"Stopped resource monitoring process (PID: {self.monitor.pid}, "
+            f"exit code: {self.monitor.exitcode})"
+        )
 
     def __exit__(self, *_) -> None:
         """Stop the monitoring process."""
