@@ -1,7 +1,7 @@
 # SPDX-FileCopyrightText: Copyright © 2023 Idiap Research Institute <contact@idiap.ch>
 #
 # SPDX-License-Identifier: GPL-3.0-or-later
-"""Test code for datasets."""
+"""Test code for database splits."""
 
 from mednet.data.split import JSONDatabaseSplit
 
